/*
 * Copyright (c) 2007 by Thoralt Franz
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 */

#include "dds.h"
#include <avr/pgmspace.h>
#include <stdio.h>
#include <math.h>
#include "main.h"
#include "panel.h"
#include "encoder.h"
#include "lcd.h"
#include "dds-hw.h"
#include "timer.h"

/* Move the ouput one 1 line */
#ifdef COMPILE_WITH_DISPLAY204
	#define DISPLAY_LINE_0 1
	#define DISPLAY_LINE_1 2
#else
	#define DISPLAY_LINE_0 0
	#define DISPLAY_LINE_1 1
#endif

//----------------------------------------------------------------------------
// menu items
//----------------------------------------------------------------------------
// +-->Freq
// |   |
// |   V
// |   Level
// |   |
// |   V
// |   Offset
// |   |
// |   V
// ^   Waveform--->Logic Hi<---+
// |   | ^         |           |
// |   | |         V           |
// ^   | |         Logic Lo    |
// |   | |         |           |
// |   | |         V           |
// |   | +-------<-back->------+
// |   V
// ^   Burst------>On Time<----+
// |   | ^         |           |
// |   | |         V           |
// ^   | |         Off Time    |
// |   | |         |           |
// |   | |         V           |
// |   | +-------<-back->------+
// |   V
// ^   Sweep------>Sweep Mode<-+
// |   | ^         |           |
// |   | |         V           |
// ^   | |         Start Freq  |
// |   | ^         |           |
// |   | |         V           |
// ^   | |         End Freq    |
// |   | ^         |           |
// |   | |         V           |
// |   | +-------<-back->------+
// |   V
// ^   AM--------->AM On<------+
// |   | ^         |           |
// |   | |         V           |
// |   | +-------<-back->------+
// |   V
// ^   FM--------->FM On<------+
// |   | ^         |           |
// |   | |         V           |
// |   | +-------<-back->------+
// |   V
// ^   TRMSC------>Range<------+
// |   | ^         |           |
// |   | |         V           |
// ^   | |         RMS         |
// |   | |         |           |
// |   | |         V           |
// ^   | |         Peak        |
// |   | |         |           |
// |   | |         V           |
// ^   | |         Permanent   |
// |   | |         |           |
// |   | |         V           |
// |   | +-------<-back->------+
// |   V
// ^   Settings--->Save<-------+
// |   | ^         |           |
// |   | |         V           |
// ^   | |         Permanent Dl|
// |   | |         |           |
// |   | |         V           |
// ^   | |         Scrollspeed |
// |   | |         |           |
// |   | |         V           |
// |   | +-------<-back->------+
// |   V
// +-<-+
//
//............................................................................
// main menu
//............................................................................
menu_entry_t PROGMEM MenuFrequency = {
		.cName = "Frequency",
		.pEncoderFunction = FrequencyEncoderFunction,
		.pEnterFunction = FrequencyEnterFunction,
		.pUpdateDisplayFunction = FrequencyUpdateDisplayFunction,
		.pSubmenu = null,
		.pPrevious = (void*)&MenuSettings,
		.pNext = (void*)&MenuLevel,
		.iEI1 = 250, .iEI2 = 50, .iEI3 = 10, .iEI4 = 1,
		.pParam = &(Params.dFrequency), .ucIdentifier = MENU_FREQUENCY};

menu_entry_t PROGMEM MenuLevel = {
		.cName = "Level",
		.pEncoderFunction = LevelEncoderFunction,
		.pEnterFunction = SwitchDBVoltsFunction,
		.pUpdateDisplayFunction = LevelUpdateDisplayFunction,
		.pSubmenu = null,
		.pPrevious = (void*)&MenuFrequency,
		.pNext = (void*)&MenuPeakLevel,
		.iEI1 = 250, .iEI2 = 50, .iEI3 = 10, .iEI4 = 1,
		.pParam = &(Params.dLevel), .ucIdentifier = MENU_LEVEL_RMS};

menu_entry_t PROGMEM MenuPeakLevel = {
		.cName = "PeakPeakLevel",
		.pEncoderFunction = LevelEncoderFunction,
		.pEnterFunction = null,
		.pUpdateDisplayFunction = PeakLevelUpdateDisplayFunction,
		.pSubmenu = null,
		.pPrevious = (void*)&MenuLevel,
		.pNext = (void*)&MenuOffset,
		.iEI1 = 250, .iEI2 = 50, .iEI3 = 10, .iEI4 = 1,
		.pParam = &(Params.dPeakLevel), .ucIdentifier = MENU_LEVEL_PEAK};

menu_entry_t PROGMEM MenuOffset = {
		.cName = "Offset",
		.pEncoderFunction = OffsetEncoderFunction,
		.pEnterFunction = OffsetEnterFunction,
		.pUpdateDisplayFunction = OffsetUpdateDisplayFunction,
		.pSubmenu = null,
		.pPrevious = (void*)&MenuPeakLevel,
		.pNext = (void*)&MenuWaveform,
		.iEI1 = 200, .iEI2 = 50, .iEI3 = 10, .iEI4 = 5,
		.pParam = null, .ucIdentifier = MENU_OFFSET};

menu_entry_t PROGMEM MenuWaveform = {
		.cName = "Waveform",
		.pEncoderFunction = UCEncoderFunction,
		.pEnterFunction = WaveformEnterFunction,
		.pUpdateDisplayFunction = WaveformUpdateDisplayFunction,
		.pSubmenu = null,
		.pPrevious = (void*)&MenuOffset,
		.pNext = (void*)&MenuSweep,
		.iEI1 = 1, .iEI2 = 1, .iEI3 = 1, .iEI4 = 1,
		.pParam = &(Params.ucWaveForm), .ucIdentifier = MENU_WAVEFORM};

menu_entry_t PROGMEM MenuSweep = {
		.cName = "Sweep",
		.pEncoderFunction = UCEncoderFunction,
		.pEnterFunction = null,
		.pUpdateDisplayFunction = SweepMenuUpdateDisplayFunction,
		.pSubmenu = (void*)&SubmenuSweepStart,
		.pPrevious = (void*)&MenuWaveform,
		.pNext = (void*)&MenuBurst,
//		.pNext = (void*)&MenuAM,
		.iEI1 = 1, .iEI2 = 1, .iEI3 = 1, .iEI4 = 1,
		.pParam = &(Params.ucSweepMenu), .ucIdentifier = MENU_SWEEP};

/*
menu_entry_t PROGMEM MenuAM = {
		.cName = "AM",
		.pEncoderFunction = null,
		.pEnterFunction = null,
		.pUpdateDisplayFunction = null,
		.pSubmenu = (void*)&SubmenuAMOn,
		.pPrevious = (void*)&MenuSweep,
		.pNext = (void*)&MenuFM,
		.iEI1 = 1, .iEI2 = 1, .iEI3 = 1, .iEI4 = 1,
		.pParam = null, .ucIdentifier = MENU_AM};

menu_entry_t PROGMEM MenuFM = {
		.cName = "FM",
		.pEncoderFunction = null,
		.pEnterFunction = null,
		.pUpdateDisplayFunction = null,
		.pSubmenu = (void*)&SubmenuFMOn,
		.pPrevious = (void*)&MenuAM,
		.pNext = (void*)&MenuBurst,
		.iEI1 = 1, .iEI2 = 1, .iEI3 = 1, .iEI4 = 1,
		.pParam = null, .ucIdentifier = MENU_FM};
*/
menu_entry_t PROGMEM MenuBurst = {
		.cName = "Burst",
		.pEncoderFunction = Burst_OnOffEncoderFunction,
		.pEnterFunction = null,
		.pUpdateDisplayFunction = Burst_OnOffUpdateDisplayFunction,
		.pSubmenu = (void*)&SubmenuBurstOnTime,
		.pPrevious = (void*)&MenuSweep,
		.pNext = (void*)&MenuTRMSC,
		.iEI1 = 1, .iEI2 = 1, .iEI3 = 1, .iEI4 = 1,
		.pParam = null, .ucIdentifier = MENU_BURST};

menu_entry_t PROGMEM MenuTRMSC = {
		.cName = "TRMSC",
		.pEncoderFunction = null,
		.pEnterFunction = null,
		.pUpdateDisplayFunction = null,
		.pSubmenu = (void*)&SubmenuTRMSC_RMS,
		.pPrevious = (void*)&MenuBurst,
		.pNext = (void*)&MenuSettings,
		.iEI1 = 1, .iEI2 = 1, .iEI3 = 1, .iEI4 = 1,
		.pParam = null, .ucIdentifier = MENU_TRMSC};

menu_entry_t PROGMEM MenuSettings = {
		.cName = "Settings",
		.pEncoderFunction = null,
		.pEnterFunction = null,
		.pUpdateDisplayFunction = null,
		.pSubmenu = (void*)&SubmenuSettingsSave,
		.pPrevious = (void*)&MenuTRMSC,
		.pNext = (void*)&MenuFrequency,
		.iEI1 = 1, .iEI2 = 1, .iEI3 = 1, .iEI4 = 1,
		.pParam = null, .ucIdentifier = MENU_TRMSC};

//............................................................................
// Submenu "Settings"
//............................................................................
menu_entry_t PROGMEM SubmenuSettingsSave = {
		.cName = "Save",
		.pEncoderFunction = UCEncoderFunction,
		.pEnterFunction = SettingsSaveEnterFunction,
		.pUpdateDisplayFunction = SettingsSaveUpdateDisplayFunction,
		.pSubmenu = null,
		.pPrevious = (void*)&SubmenuSettingsBack,
		.pNext = (void*)&SubmenuSettingsPermanentTime,
		.iEI1 = 1, .iEI2 = 1, .iEI3 = 1, .iEI4 = 1,
		.pParam = &(Params.ucAutosave),
		.ucIdentifier = SUBMENU_SAVE};

menu_entry_t PROGMEM SubmenuSettingsPermanentTime = {
		.cName = "Permanent TRMSC Delay",
		.pEncoderFunction = IEncoderFunction,
		.pEnterFunction = null,
		.pUpdateDisplayFunction = TimeUpdateDisplayFunction,
		.pSubmenu = null,
		.pPrevious = (void*)&SubmenuSettingsSave,
		.pNext = (void*)&SubmenuSettingsScrollSpeed,
		.iEI1 = 100, .iEI2 = 100, .iEI3 = 100, .iEI4 = 100,
		.pParam = &(Params.iPermanentTRMSC_Delay),
		.ucIdentifier = SUBMENU_PERMANENTTIME};

menu_entry_t PROGMEM SubmenuSettingsScrollSpeed = {
		.cName = "Display Scroll Speed",
		.pEncoderFunction = IEncoderFunction,
		.pEnterFunction = null,
		.pUpdateDisplayFunction = TimeUpdateDisplayFunction,
		.pSubmenu = null,
		.pPrevious = (void*)&SubmenuSettingsPermanentTime,
		.pNext = (void*)&SubmenuSettingsBack,
		.iEI1 = 100, .iEI2 = 100, .iEI3 = 25, .iEI4 = 25,
		.pParam = &(Params.iScrollSpeed),
		.ucIdentifier = SUBMENU_SCROLLSPEED};

menu_entry_t PROGMEM SubmenuSettingsBack = {
		.cName = "    back",
		.pEncoderFunction = null,
		.pEnterFunction = null,
		.pUpdateDisplayFunction = null,
		.pSubmenu = (void*)&MenuSettings,
		.pPrevious = (void*)&SubmenuSettingsScrollSpeed,
		.pNext = (void*)&SubmenuSettingsSave,
		.iEI1 = 1, .iEI2 = 1, .iEI3 = 1, .iEI4 = 1, .pParam = null,
		.ucIdentifier = SUBMENU_SETTINGSBACK};


//............................................................................
// Submenu "TRMSC"
//............................................................................
menu_entry_t PROGMEM SubmenuTRMSC_RMS = {
		.cName = "RMS",
		.pEncoderFunction = null,
		.pEnterFunction = SwitchDBVoltsFunction,
		.pUpdateDisplayFunction = TRMSC_LevelUpdateDisplayFunction,
		.pSubmenu = null,
		.pPrevious = (void*)&SubmenuTRMSC_Back,
		.pNext = (void*)&SubmenuTRMSC_Peak,
		.iEI1 = 1, .iEI2 = 1, .iEI3 = 1, .iEI4 = 1,
		.pParam = &(g_dTRMSC_RMS), .ucIdentifier = SUBMENU_TRMS_RMS};

menu_entry_t PROGMEM SubmenuTRMSC_Peak = {
		.cName = "PeakPeak",
		.pEncoderFunction = null,
		.pEnterFunction = null,
		.pUpdateDisplayFunction = TRMSC_LevelUpdateDisplayFunction,
		.pSubmenu = null,
		.pPrevious = (void*)&SubmenuTRMSC_RMS,
		.pNext = (void*)&SubmenuTRMSC_Range,
		.iEI1 = 1, .iEI2 = 1, .iEI3 = 1, .iEI4 = 1,
		.pParam = &(g_dTRMSC_Peak), .ucIdentifier = SUBMENU_TRMS_PEAK};

menu_entry_t PROGMEM SubmenuTRMSC_Range = {
		.cName = "Range",
		.pEncoderFunction = UCEncoderFunction,
		.pEnterFunction = null,
		.pUpdateDisplayFunction = TRMSC_RangeUpdateDisplayFunction,
		.pSubmenu = null,
		.pPrevious = (void*)&SubmenuTRMSC_Peak,
		.pNext = (void*)&SubmenuTRMSC_Permanent,
		.iEI1 = 1, .iEI2 = 1, .iEI3 = 1, .iEI4 = 1,
		.pParam = &(Params.ucRange),
		.ucIdentifier = SUBMENU_TRMS_RANGE};

menu_entry_t PROGMEM SubmenuTRMSC_Permanent = {
		.cName = "Permanent Display",
		.pEncoderFunction = UCEncoderFunction,
		.pEnterFunction = null,
		.pUpdateDisplayFunction = TRMSC_PermanentUpdateDisplayFunction,
		.pSubmenu = null,
		.pPrevious = (void*)&SubmenuTRMSC_Range,
		.pNext = (void*)&SubmenuTRMSC_Back,
		.iEI1 = 1, .iEI2 = 1, .iEI3 = 1, .iEI4 = 1,
		.pParam = &(Params.ucPermanentTRMSC),
		.ucIdentifier = SUBMENU_TRMS_PERMANENT};

menu_entry_t PROGMEM SubmenuTRMSC_Back = {
		.cName = "    back",
		.pEncoderFunction = null,
		.pEnterFunction = null,
		.pUpdateDisplayFunction = null,
		.pSubmenu = (void*)&MenuTRMSC,
		.pPrevious = (void*)&SubmenuTRMSC_Permanent,
		.pNext = (void*)&SubmenuTRMSC_RMS,
		.iEI1 = 1, .iEI2 = 1, .iEI3 = 1, .iEI4 = 1, .pParam = null,
		.ucIdentifier = SUBMENU_TRMS_BACK};

//............................................................................
// Submenu "Logic"
//............................................................................
menu_entry_t PROGMEM SubmenuLogicHi = {
		.cName = "Logic Hi",
		.pEncoderFunction = LogicLevelEncoderFunction,
		.pEnterFunction = LogicLevelEnterFunction,
		.pUpdateDisplayFunction = LogicLevelUpdateDisplayFunction,
		.pSubmenu = null,
		.pPrevious = (void*)&SubmenuLogicBack,
		.pNext = (void*)&SubmenuLogicLo,
		.iEI1 = 250, .iEI2 = 50, .iEI3 = 10, .iEI4 = 5,
		.pParam = &(Params.iLogicHi), .ucIdentifier = SUBMENU_LOGIC_HIGH};

menu_entry_t PROGMEM SubmenuLogicLo = {
		.cName = "Logic Lo",
		.pEncoderFunction = LogicLevelEncoderFunction,
		.pEnterFunction = LogicLevelEnterFunction,
		.pUpdateDisplayFunction = LogicLevelUpdateDisplayFunction,
		.pSubmenu = null,
		.pPrevious = (void*)&SubmenuLogicHi,
		.pNext = (void*)&SubmenuLogicBack,
		.iEI1 = 250, .iEI2 = 50, .iEI3 = 10, .iEI4 = 5,
		.pParam = &(Params.iLogicLo), .ucIdentifier = SUBMENU_LOGIC_LOW};

menu_entry_t PROGMEM SubmenuLogicBack = {
		.cName = "    back",
		.pEncoderFunction = null,
		.pEnterFunction = null,
		.pUpdateDisplayFunction = null,
		.pSubmenu = (void*)&MenuWaveform,
		.pPrevious = (void*)&SubmenuLogicLo,
		.pNext = (void*)&SubmenuLogicHi,
		.iEI1 = 1, .iEI2 = 1, .iEI3 = 1, .iEI4 = 1, .pParam = null,
		.ucIdentifier = SUBMENU_LOGIC_BACK};

//............................................................................
// Submenu "Sweep"
//............................................................................
menu_entry_t PROGMEM SubmenuSweepStart = {
		.cName = "Startfrq",
		.pEncoderFunction = FrequencyEncoderFunction,
		.pEnterFunction = null,
		.pUpdateDisplayFunction = FrequencyUpdateDisplayFunction,
		.pSubmenu = null,
		.pPrevious = (void*)&SubmenuSweepBack,
		.pNext = (void*)&SubmenuSweepEnd,
		.iEI1 = 250, .iEI2 = 50, .iEI3 = 10, .iEI4 = 1,
		.pParam = &(Params.dSweepStart), .ucIdentifier = SUBMENU_SWEEP_START};

menu_entry_t PROGMEM SubmenuSweepEnd = {
		.cName = "Endfreq",
		.pEncoderFunction = FrequencyEncoderFunction,
		.pEnterFunction = null,
		.pUpdateDisplayFunction = FrequencyUpdateDisplayFunction,
		.pSubmenu = null,
		.pPrevious = (void*)&SubmenuSweepStart,
		.pNext = (void*)&SubmenuSweepCenter,
		.iEI1 = 250, .iEI2 = 50, .iEI3 = 10, .iEI4 = 1,
		.pParam = &(Params.dSweepEnd), .ucIdentifier = SUBMENU_SWEEP_END};

menu_entry_t PROGMEM SubmenuSweepCenter = {
		.cName = "Centerfrq",
		.pEncoderFunction = FrequencyEncoderFunction,
		.pEnterFunction = null,
		.pUpdateDisplayFunction = FrequencyUpdateDisplayFunction,
		.pSubmenu = null,
		.pPrevious = (void*)&SubmenuSweepEnd,
		.pNext = (void*)&SubmenuSweepSpanFactor,
		.iEI1 = 250, .iEI2 = 50, .iEI3 = 10, .iEI4 = 1,
		.pParam = &(Params.dSweepCenter), .ucIdentifier = SUBMENU_SWEEP_CENTER};

menu_entry_t PROGMEM SubmenuSweepSpanFactor = {
		.cName = "Frq Span",
		.pEncoderFunction = SweepDoubleEncoderFunction,
		.pEnterFunction = null,
		.pUpdateDisplayFunction = SweepDoubleUpdateDisplayFunction,
		.pSubmenu = null,
		.pPrevious = (void*)&SubmenuSweepCenter,
		.pNext = (void*)&SubmenuSweepTime,
		.iEI1 = 10, .iEI2 = 10, .iEI3 = 10, .iEI4 = 1,
		.pParam = &(Params.dSweepSpanFactor), .ucIdentifier = SUBMENU_SWEEP_SPANFACTOR};

menu_entry_t PROGMEM SubmenuSweepTime = {
		.cName = "Swp Time",
		.pEncoderFunction = IEncoderFunction,
		.pEnterFunction = null,
		.pUpdateDisplayFunction = TimeUpdateDisplayFunction,
		.pSubmenu = null,
		.pPrevious = (void*)&SubmenuSweepSpanFactor,
		.pNext = (void*)&SubmenuSweepMode,
		.iEI1 = 1000, .iEI2 = 500, .iEI3 = 200,  .iEI4 = 100,
//		.iEI1 = SWEEP_UPDATE_INTERVAL*100, .iEI2 = SWEEP_UPDATE_INTERVAL*10,
//		.iEI3 = SWEEP_UPDATE_INTERVAL*2, .iEI4 = SWEEP_UPDATE_INTERVAL,
		.pParam = &(Params.iSweepTime), .ucIdentifier = SUBMENU_SWEEP_TIME};

menu_entry_t PROGMEM SubmenuSweepMode = {
		.cName = "Swp Mode",
		.pEncoderFunction = UCEncoderFunction,
		.pEnterFunction = null,
		.pUpdateDisplayFunction = SweepModeUpdateDisplayFunction,
		.pSubmenu = null,
		.pPrevious = (void*)&SubmenuSweepTime,
		.pNext = (void*)&SubmenuSweepSlope,
		.iEI1 = 1, .iEI2 = 1, .iEI3 = 1, .iEI4 = 1,
		.pParam = &(Params.ucSweepMode), .ucIdentifier = SUBMENU_SWEEP_MODE};

menu_entry_t PROGMEM SubmenuSweepSlope = {
		.cName = "Sw Slope",
		.pEncoderFunction = UCEncoderFunction,
		.pEnterFunction = null,
		.pUpdateDisplayFunction = SweepSlopeUpdateDisplayFunction,
		.pSubmenu = null,
		.pPrevious = (void*)&SubmenuSweepMode,
		.pNext = (void*)&SubmenuSweepMarker,
		.iEI1 = 1, .iEI2 = 1, .iEI3 = 1, .iEI4 = 1,
		.pParam = &(Params.ucSweepSlope), .ucIdentifier = SUBMENU_SWEEP_SLOPE};

menu_entry_t PROGMEM SubmenuSweepMarker = {
		.cName = "Marker",
		.pEncoderFunction = UCEncoderFunction,
		.pEnterFunction = null,
		.pUpdateDisplayFunction = SweepMarkerUpdateDisplayFunction,
		.pSubmenu = null,
		.pPrevious = (void*)&SubmenuSweepSlope,
		.pNext = (void*)&SubmenuSweepMarkerMode,
		.iEI1 = 1, .iEI2 = 1, .iEI3 = 1, .iEI4 = 1,
		.pParam = &(Params.ucSweepMarker), .ucIdentifier = SUBMENU_SWEEP_MARKER};

menu_entry_t PROGMEM SubmenuSweepMarkerMode = {
		.cName = "MarkerMode",
		.pEncoderFunction = UCEncoderFunction,
		.pEnterFunction = null,
		.pUpdateDisplayFunction = SweepMarkerModeUpdateDisplayFunction,
		.pSubmenu = null,
		.pPrevious = (void*)&SubmenuSweepMarker,
		.pNext = (void*)&SubmenuSweepMarkerHeight,
		.iEI1 = 1, .iEI2 = 1, .iEI3 = 1, .iEI4 = 1,
		.pParam = &(Params.ucSweepMarkerMode), .ucIdentifier = SUBMENU_SWEEP_MARKERMODE};

menu_entry_t PROGMEM SubmenuSweepMarkerHeight = {
		.cName = "MarkerHeight",
		.pEncoderFunction = SweepDoubleEncoderFunction,
		.pEnterFunction = null,
		.pUpdateDisplayFunction = SweepDoubleUpdateDisplayFunction,
		.pSubmenu = null,
		.pPrevious = (void*)&SubmenuSweepMarkerMode,
		.pNext = (void*)&SubmenuSweepBack,
		.iEI1 = 1, .iEI2 = 1, .iEI3 = 1, .iEI4 = 1,
		.pParam = &(Params.dSweepMarkerHeight), .ucIdentifier = SUBMENU_SWEEP_MARKERHGHT};

menu_entry_t PROGMEM SubmenuSweepBack = {
		.cName = "    back",
		.pEncoderFunction = null,
		.pEnterFunction = null,
		.pUpdateDisplayFunction = null,
		.pSubmenu = (void*)&MenuSweep,
		.pPrevious = (void*)&SubmenuSweepMarkerHeight,
		.pNext = (void*)&SubmenuSweepStart,
		.iEI1 = 1, .iEI2 = 1, .iEI3 = 1, .iEI4 = 1,
		.pParam = null, .ucIdentifier = SUBMENU_SWEEP_BACK};

//............................................................................
// Submenu "Burst"
//............................................................................
menu_entry_t PROGMEM SubmenuBurstOnTime = {
		.cName = "On Time",
		.pEncoderFunction = IEncoderFunction,
		.pEnterFunction = null,
		.pUpdateDisplayFunction = TimeUpdateDisplayFunction,
		.pSubmenu = null,
		.pPrevious = (void*)&SubmenuBurstBack,
		.pNext = (void*)&SubmenuBurstOffTime,
		.iEI1 = 250, .iEI2 = 50, .iEI3 = 10, .iEI4 = 1,
		.pParam = &(Params.iBurst1), .ucIdentifier = SUBMENU_BURST1};

menu_entry_t PROGMEM SubmenuBurstOffTime = {
		.cName = "Off Time",
		.pEncoderFunction = IEncoderFunction,
		.pEnterFunction = null,
		.pUpdateDisplayFunction = TimeUpdateDisplayFunction,
		.pSubmenu = null,
		.pPrevious = (void*)&SubmenuBurstOnTime,
		.pNext = (void*)&SubmenuBurstBack,
		.iEI1 = 250, .iEI2 = 50, .iEI3 = 10, .iEI4 = 1,
		.pParam = &(Params.iBurst0), .ucIdentifier = SUBMENU_BURST0};

menu_entry_t PROGMEM SubmenuBurstBack = {
		.cName = "    back",
		.pEncoderFunction = null,
		.pEnterFunction = null,
		.pUpdateDisplayFunction = null,
		.pSubmenu = (void*)&MenuBurst,
		.pPrevious = (void*)&SubmenuBurstOffTime,
		.pNext = (void*)&SubmenuBurstOnTime,
		.iEI1 = 1, .iEI2 = 1, .iEI3 = 1, .iEI4 = 1, .pParam = null,
		.ucIdentifier = SUBMENU_BURST_BACK};

//............................................................................
// Submenu "AM"
//............................................................................
/*
menu_entry_t PROGMEM SubmenuAMOn = {
		.cName = "AM On",
		.pEncoderFunction = null,
		.pEnterFunction = null,
		.pUpdateDisplayFunction = null,
		.pSubmenu = null,
		.pPrevious = (void*)&SubmenuAMBack,
		.pNext = (void*)&SubmenuAMBack,
		.iEI1 = 1, .iEI2 = 1, .iEI3 = 1, .iEI4 = 1, .pParam = null};

menu_entry_t PROGMEM SubmenuAMBack = {
		.cName = "back",
		.pEncoderFunction = null,
		.pEnterFunction = null,
		.pUpdateDisplayFunction = null,
		.pSubmenu = (void*)&MenuAM,
		.pPrevious = (void*)&SubmenuAMOn,
		.pNext = (void*)&SubmenuAMOn,
		.iEI1 = 1, .iEI2 = 1, .iEI3 = 1, .iEI4 = 1, .pParam = null};
*/
//............................................................................
// Submenu "FM"
//............................................................................
/*
menu_entry_t PROGMEM SubmenuFMOn = {
		.cName = "FM On",
		.pEncoderFunction = null,
		.pEnterFunction = null,
		.pUpdateDisplayFunction = null,
		.pSubmenu = null,
		.pPrevious = (void*)&SubmenuFMBack,
		.pNext = (void*)&SubmenuFMBack,
		.iEI1 = 1, .iEI2 = 1, .iEI3 = 1, .iEI4 = 1, .pParam = null};

menu_entry_t PROGMEM SubmenuFMBack = {
		.cName = "    back",
		.pEncoderFunction = null,
		.pEnterFunction = null,
		.pUpdateDisplayFunction = null,
		.pSubmenu = (void*)&MenuFM,
		.pPrevious = (void*)&SubmenuFMOn,
		.pNext = (void*)&SubmenuFMOn,
		.iEI1 = 1, .iEI2 = 1, .iEI3 = 1, .iEI4 = 1, .pParam = null};
*/

//----------------------------------------------------------------------------
// globals
//----------------------------------------------------------------------------

// this variable holds a pointer to the current menu item
menu_entry_t *g_currentMenuItem = (void*)&MenuFrequency;

// this flag is set to "1" everytime the menu changes, jobPanel() will then
// show the new menu item on LCD
uint8_t g_ucMenuItemChanged = 1;

// variable to hold the previous button state
uint8_t g_uLastButtons = 0;

// current scrolling positions
uint8_t g_ucScrollOffset = 0;

#ifdef ENCODER_NAVIGATION
// variable for the turning encoder menu marker
uint8_t g_ucEncoderMenuMarker = 0;
#endif

uint8_t g_ucFrequencyMode = FREQUENCY_CONT;


//----------------------------------------------------------------------------
// jobPanel
//
// Read buttons and encoder input, change appropriate parameters, write to
// LCD.
//
// This function is called repeatedly.
//
// -> --
// <- --
//----------------------------------------------------------------------------
void jobPanel(void)
{
	uint8_t uButtons, i, j, ucScrollLength;
	int16_t iEncoderPos;
	uint16_t u;
	menu_entry_t *m;
	char s[9];

	// variables to receive function pointers
	void (*EncoderFunction)(int16_t iEncoderValue, void *pParam);
	void (*EnterFunction)(void);
	void (*UpdateDisplayFunction)(uint8_t ucForceUpdate, void *pParam);

/*	uncomment this to test encoder acceleration
	iEncoderPos = GetAndResetEncPos();
	SetEncoderIncrements(4, 3, 2, 1);
	if(iEncoderPos)
	{
		printf_P(PSTR("%8i\n"), iEncoderPos);
	};
	return;
*/

#ifdef ENCODER_NAVIGATION
	// reset menu mode to normal if necessary
	if(0==g_uEncoderMenuTimer) g_ucEncoderMenuMarker = 0;
#endif

	// check buttons (only if button bounce timer reached zero)
	ATOMIC_RW(u, xg_uButtonTimer);
	if(0==u)
	{
		uButtons = LCDGetButton();

#ifdef FIRMWARE_UPDATE
		// firmware update mode if all buttons are pressed
	    if((uButtons&(BUTTON_ENTER|BUTTON_UP|BUTTON_DOWN))
	    		==(BUTTON_ENTER|BUTTON_UP|BUTTON_DOWN))
		{
			LCDOverwrite_P(0, DISPLAY_LINE_0, 8, PSTR("FIRMWARE"));
			LCDOverwrite_P(0, DISPLAY_LINE_1, 8, PSTR(" UPDATE "));

			// jump to flashloader
			asm volatile("ldi r30,0x00\n");
			asm volatile("ldi r31,0x3F\n");
			asm volatile("ijmp\n");
		}
#endif

#ifdef COMPILE_WITH_DISPLAY204
        // BUTTON_SOFT_KEY_1? (and previous not BUTTON_SOFT_KEY_1)
        if ((uButtons & BUTTON_SOFT_KEY_1) && (!(g_uLastButtons & BUTTON_SOFT_KEY_1)))
        {
            ATOMIC_RW(xg_uButtonTimer, BUTTON_TIME);

            // switch off TRMSC for a short time
            ATOMIC_RW(xg_uPermanentTRMSC_Timer, (Params.iPermanentTRMSC_Delay<<1)); //+
			// reset to normal mode
			initParams();
        }
#endif

	    // up? (and previous not up)
		if((uButtons&BUTTON_UP)&&(!(g_uLastButtons&BUTTON_UP)))
		{
			ATOMIC_RW(xg_uButtonTimer, BUTTON_TIME);

			// switch off TRMSC for a short time
			ATOMIC_RW(xg_uPermanentTRMSC_Timer,
					(Params.iPermanentTRMSC_Delay<<1)); //+

			// if possible, go back to previous menu item
			m = (void*)pgm_read_word(&g_currentMenuItem->pPrevious);
			if(null!=m)
			{
				g_currentMenuItem = m;
				g_ucMenuItemChanged = 1;
			}

#ifdef ENCODER_NAVIGATION
			// switch on encoder menu mode
			g_ucEncoderMenuMarker = 1;
			g_uEncoderMenuTimer = ENCODER_MENU_TIME;
#endif
		}

		// down? (and previous not down)
		if((uButtons&BUTTON_DOWN)&&(!(g_uLastButtons&BUTTON_DOWN)))
		{
			ATOMIC_RW(xg_uButtonTimer, BUTTON_TIME);

			// switch off TRMSC for a short time
			ATOMIC_RW(xg_uPermanentTRMSC_Timer,
					(Params.iPermanentTRMSC_Delay<<1)); //+

			// if possible, advance to next menu item
			m = (void*)pgm_read_word(&g_currentMenuItem->pNext);
			if(null!=m)
			{
				g_currentMenuItem = m;
				g_ucMenuItemChanged = 1;
			}

#ifdef ENCODER_NAVIGATION
			// switch on encoder menu mode
			g_ucEncoderMenuMarker = 1;
			g_uEncoderMenuTimer = ENCODER_MENU_TIME;
#endif
		}

		// enter? (and previous not enter)
		if((uButtons&BUTTON_ENTER)&&(!(g_uLastButtons&BUTTON_ENTER)))
		{
			ATOMIC_RW(xg_uButtonTimer, BUTTON_TIME);

			ATOMIC_RW(xg_uPermanentTRMSC_Timer,
					(Params.iPermanentTRMSC_Delay<<1)); //+

			EnterFunction =
				(void*)pgm_read_word(&g_currentMenuItem->pEnterFunction);

#ifdef ENCODER_NAVIGATION
			// if encoder menu changing was on, reset it now and do
			// not start anything yet
			if(g_ucEncoderMenuMarker)
			{
				g_ucEncoderMenuMarker = 0;
			}
			else
#endif
			{
				// if a submenu was defined, switch to this submenu
				m = (void*)pgm_read_word(&g_currentMenuItem->pSubmenu);
				if(null!=m)
				{
					g_currentMenuItem = m;
					g_ucMenuItemChanged = 1;
				}
				else
				// if an "Enter" function was defined, call it now
				{
					if(null!=EnterFunction) EnterFunction();
				}
			}
		}

		// save current button state
		g_uLastButtons = uButtons;
	}

	// Update Parameter for DSP-command
	LastParams.ucDisplayedMenu = Params.ucDisplayedMenu =
		pgm_read_byte(&g_currentMenuItem->ucIdentifier); //0

	// get the encoder function and update function
	EncoderFunction =
		(void*)pgm_read_word(&g_currentMenuItem->pEncoderFunction);
	UpdateDisplayFunction =
		(void*)pgm_read_word(&g_currentMenuItem->pUpdateDisplayFunction);

	// display new menu item if necessary and set new encoder
	// acceleration values
	if(g_ucMenuItemChanged)
	{
		SetActivityTimer(75);
		g_ucMenuItemChanged = 0;

		// force output parameter name to LCD
		ATOMIC_RW(xg_uScrollTimer, 0xFFFF);
		g_ucScrollOffset = 0;

		// set new acceleration values
		SetEncoderAcceleration(pgm_read_word(&g_currentMenuItem->iEI1),
				pgm_read_word(&g_currentMenuItem->iEI2),
				pgm_read_word(&g_currentMenuItem->iEI3),
				pgm_read_word(&g_currentMenuItem->iEI4));

		// also update parameter display
		if(null!=UpdateDisplayFunction)
			UpdateDisplayFunction(1,
					(void*)pgm_read_word(&g_currentMenuItem->pParam));

		// if there's no EncoderFunction, display triangle to indicate if there
		// is a submenu or output a blank line if not
		else
		{
			if(null!=(void*)pgm_read_word(&g_currentMenuItem->pSubmenu))
			{
#ifdef COMPILE_WITH_DISPLAY204
			LCDOverwrite_P(7, 2, 1, PSTR("\x10"));
#else
			LCDOverwrite_P(7, 1, 1, PSTR("\x05"));
#endif
				

			}
			else
			{
				LCDClearLine(DISPLAY_LINE_1);
			}
		}
	}

	// do text scrolling
	ATOMIC_RW(u, xg_uScrollTimer);
	if((0==u)||(u>0xF000))
	{
		// if text has been reset, the timer is counting downwards from 0xFFFF
		// -> wait longer if the text has just been updated, this causes
		// a small delay before scrolling begins
		if(u>0xF000)
		{
			ATOMIC_RW(xg_uScrollTimer, (Params.iScrollSpeed<<1)*4);
		}
		else
		{
			ATOMIC_RW(xg_uScrollTimer, (Params.iScrollSpeed<<1)); //+
		}

#ifdef ENCODER_NAVIGATION
		ucScrollLength = (0==g_ucEncoderMenuMarker) ? SPALTEN_MAX : ((SPALTEN_MAX) - 1));
#else
		ucScrollLength = SPALTEN_MAX;
#endif
		// do not scroll if length < 9 (or 8, if encoder scrolling is active)
		if(strlen_P(g_currentMenuItem->cName)<=ucScrollLength)
		{
			// output menu item
			LCDOverwrite_P(0, DISPLAY_LINE_0, strlen_P(g_currentMenuItem->cName), g_currentMenuItem->cName);
		}
		else
		{
			// create scrolling text for LCD
			j = g_ucScrollOffset;
			for(i=0; i<ucScrollLength; i++)
			{
				if(j>=(strlen_P(g_currentMenuItem->cName)+SPALTEN_MAX)) j = 0;

				if(j<strlen_P(g_currentMenuItem->cName))
					s[i] = pgm_read_byte(g_currentMenuItem->cName + j);
				else s[i] = 0x020;
				j++;
			}

			LCDWrite(0, DISPLAY_LINE_0, ucScrollLength, s);


			g_ucScrollOffset++;
			// add 8 spaces at the end
			if(g_ucScrollOffset>=(strlen_P(g_currentMenuItem->cName)+SPALTEN_MAX))
				g_ucScrollOffset = 0;
		}
	}

#ifdef ENCODER_NAVIGATION
	// show encoder turn symbol?
	if(g_ucEncoderMenuMarker&&(0==g_uTurnSymbolTimer))
	{
		g_uTurnSymbolTimer = 500;
		s[0] = g_ucEncoderMenuMarker - 1;
		if(++g_ucEncoderMenuMarker>4) g_ucEncoderMenuMarker = 1;
		LCDWrite(7, DISPLAY_LINE_0, 1, s);
	}
#endif

	// call the function which updates the Display for the current parameter
	// LCD will only be written if parameter changed
	if(null!=UpdateDisplayFunction)
		UpdateDisplayFunction(0,
			(void*)pgm_read_word(&g_currentMenuItem->pParam));

	// read encoder position
	// current parameter is changed if necessary
	iEncoderPos = GetAndResetEncPos();
	if(iEncoderPos)
	{
		// reset timer for permanent TRMSC display to allow parameter
		// display for a short time
		ATOMIC_RW(xg_uPermanentTRMSC_Timer,
				(Params.iPermanentTRMSC_Delay<<1)); //+

#ifdef ENCODER_NAVIGATION
		// if in encoder menu mode, use the encoder to scroll through
		// menu
		if(g_ucEncoderMenuMarker)
		{
			if(iEncoderPos<0)
			{
				// if possible, go back to previous menu item
				m = (void*)pgm_read_word(&g_currentMenuItem->pPrevious);
				if(null!=m)
				{
					g_currentMenuItem = m;
					g_ucMenuItemChanged = 1;
				}
			}
			else
			{
				// if possible, advance to next menu item
				m = (void*)pgm_read_word(&g_currentMenuItem->pNext);
				if(null!=m)
				{
					g_currentMenuItem = m;
					g_ucMenuItemChanged = 1;
				}
			}
			g_uEncoderMenuTimer = ENCODER_MENU_TIME;
		}
		// use the encoder to change values
		else
#endif
		{
			// call encoder function if it is not null
			if(null!=EncoderFunction)
				EncoderFunction(iEncoderPos,
						(void*)pgm_read_word(&g_currentMenuItem->pParam));
				CheckLimits();
			if(null!=UpdateDisplayFunction)
 				UpdateDisplayFunction(0,
					(void*)pgm_read_word(&g_currentMenuItem->pParam));

		}
	}

	// display permanent TRMSC value if necessary
	ATOMIC_RW(u, xg_uPermanentTRMSC_Timer);
	if((0==u) && Params.ucPermanentTRMSC && ((g_currentMenuItem != (void*)&SubmenuTRMSC_RMS) && (g_currentMenuItem != (void*)&SubmenuTRMSC_Peak)))
	{
		if(PERMANENT_TRMSC_RMS==Params.ucPermanentTRMSC)
			TRMSC_LevelUpdateDisplayFunction(1, &g_dTRMSC_RMS);
		else TRMSC_LevelUpdateDisplayFunction(1, &g_dTRMSC_Peak);
	}
	Panel_UpdateStatusLine();
}


//----------------------------------------------------------------------------
// RoundPrecision
//
// Rounds a value to a multiple of a selectable precision
//
// -> --
// <- --
//----------------------------------------------------------------------------
void RoundPrecision(double *d, double prec)
{
	int16_t i;

	i = (*d / prec) + 0.5f;
	*d = i * prec;
}

//----------------------------------------------------------------------------
// TerzDecode
//
// Converts Terz frequency codes to double values
//
// -> --
// <- --
//----------------------------------------------------------------------------
double TerzDecode(uint16_t code)
{
// Coding for frequencies: (ui_TerzArray[x]>>4 ) * 10 ^ ( int16_t)(ui_TerzArray[x] & 0x000F ) - 2)
// 3 MSB nibbles: frequency f
// LSB nibble: exponent e  (with offset 2)
// frequency = f*10^(e-2)

  	return  (double) ( code >> 4 ) *  pow( 10 , ( (int8_t)( code & 0x000F ) - 2 )) ;
}

//----------------------------------------------------------------------------
// FrequencyEncoderFunction
//
// Gets called when encoder is turned while frequency menu item (or sweep
// start/end frequency submenu item) was active
//
// -> iEncoderPos = encoder value difference since last call (positive or
//                  negative)
//    pParam: --
// <- --
//----------------------------------------------------------------------------
void FrequencyEncoderFunction(int16_t iEncoderPos, void *pParam)
{
	double *d = (double*)pParam;
	double dIncrement;
	uint16_t c;

	if (g_ucFrequencyMode == FREQUENCY_CONT)
	{


	if ( iEncoderPos >= 0)  // to make range transistions more smooth, but wait for more memory
	{
		// scale the increment/decrement appropriately
		if(*d<100.0f)
			dIncrement = 0.01f;
		else if(*d<1000.0f)
			dIncrement = 0.1f;
		else if(*d<10000.0f)
			dIncrement = 1.0f;
		else
			dIncrement = 10.0f;
	}
	else
	{
		if(*d<=100.05f)
			dIncrement = 0.01f;
		else if(*d<=1000.5f)
			dIncrement = 0.1f;
		else if(*d<=10005.0f)
			dIncrement = 1.0f;
		else
			dIncrement = 10.0f;
	}


	*d += (double)iEncoderPos * dIncrement;

	RoundPrecision(d, dIncrement);

	}
	else // Terz-Mode
	{
		if (iEncoderPos >= 0)
		{
			if ((c = pgm_read_word(&g_uiTerzArray[gc_TerzNum + 1])) != 0)
			{
				gc_TerzNum++;
			}
		}
		else
		{
			if (gc_TerzNum >= 1)
			{
				gc_TerzNum--;
			}
		}

	*d = TerzDecode(pgm_read_word(&g_uiTerzArray[gc_TerzNum]));

	}
}


//----------------------------------------------------------------------------
// SweepDoubleEncoderFunction
//
// Gets called when encoder is turned while frequency menu item (or sweep
// start/end frequency submenu item) was active
//
// -> iEncoderPos = encoder value difference since last call (positive or
//                  negative)
//    pParam: --
// <- --
//----------------------------------------------------------------------------
void SweepDoubleEncoderFunction(int16_t iEncoderPos, void *pParam)
{
	double *d = (double*)pParam;

	// scale the increment/decrement appropriately

	*d += (double)iEncoderPos * 0.01f;
}

//----------------------------------------------------------------------------
// FrequencyUpdateDisplayFunction
//
// Gets called periodically to update frequency display
//
// -> pParam: --
// <- --
//----------------------------------------------------------------------------
void FrequencyUpdateDisplayFunction(uint8_t ucForceUpdate, void *pParam)
{
	double *d = (double*)pParam;

	// check all frequency values for update because this function displays
	// main frequency, sweep start and sweep end frequency
	if((pLastParams->dFrequency!=pParams->dFrequency)||
		(pLastParams->dSweepStart!=pParams->dSweepStart)||
		(pLastParams->dSweepEnd!=pParams->dSweepEnd)||
		(pLastParams->dSweepCenter!=pParams->dSweepCenter)||
		ucForceUpdate)
	{
		// output parameter to LCD
		PrintLCD(*d, 0, LCD_TYPE_FREQUENCY);
	}
}

//----------------------------------------------------------------------------
// FrequencyEnterFunction
//
// Gets called when encoder is pushed while waveform menu item was active
//
// -> --
// <- --
//----------------------------------------------------------------------------
void FrequencyEnterFunction()
{
	uint8_t  i = 0;
	uint16_t c;

	// toggle frequency selection between continuous selection and terz selections.
	if (g_ucFrequencyMode == FREQUENCY_CONT)
	{
		g_ucFrequencyMode = FREQUENCY_TERZ;
		// based on current frequency, select next frequency from list
		while ( (c = pgm_read_word(&g_uiTerzArray[i])) != 0)
		{
			gc_TerzNum = i;

			if ( TerzDecode(c) >= Params.dFrequency)
				break;

			i++;
		}

		Params.dFrequency = TerzDecode(pgm_read_word(&g_uiTerzArray[gc_TerzNum]));
	}
	else
	{
		g_ucFrequencyMode = FREQUENCY_CONT;
	}

}

//----------------------------------------------------------------------------
// SweepDoubleUpdateDisplayFunction
//
// Gets called periodically to update frequency display
//
// -> pParam: --
// <- --
//----------------------------------------------------------------------------
void SweepDoubleUpdateDisplayFunction(uint8_t ucForceUpdate, void *pParam)
{
	double *d = (double*)pParam;

	// check all frequency values for update because this function displays
	// main frequency, sweep start and sweep end frequency
	if(	(LastParams.dSweepMarkerHeight!=Params.dSweepMarkerHeight)||
		(LastParams.dSweepSpanFactor!=Params.dSweepSpanFactor)||
		ucForceUpdate)
	{
		// output parameter to LCD
		PrintLCD(*d, 0, LCD_TYPE_FLOAT);
	}
}

//----------------------------------------------------------------------------
// LevelEncoderFunction
//
// Gets called when encoder is turned while level menu item was active
//
// -> iEncoderPos = encoder value difference since last call (positive or
//                  negative)
//    pParam: --
// <- --
//----------------------------------------------------------------------------
void LevelEncoderFunction(int16_t iEncoderPos, void *pParam )
{
	double *d = (double*)pParam;
	double dIncrement;


	if ( iEncoderPos >= 0)  // to make range transistions more smooth, but wait for more memory
	{
		// above 200 mV: use steps of 1 mV
		if( *d > 200.0f)
		{
			dIncrement = 1.0f;
		}
		// else use steps of 1/40 mV
		else
		{
			dIncrement = 0.025f;
		}

	}
	else
	{
		// above 200 mV: use steps of 1 mV
		if( *d >= 200.1f)
		{
			dIncrement = 1.0f;
		}
		// else use steps of 1/40 mV
		else
		{
			dIncrement = 0.025f;
		}

	}


	*d += (double)iEncoderPos * dIncrement;

	RoundPrecision(d, dIncrement);
}

//----------------------------------------------------------------------------
// LevelUpdateDisplayFunction
//
// Gets called periodically to update level display
//
// -> pParam: --
// <- --
//----------------------------------------------------------------------------
void LevelUpdateDisplayFunction(uint8_t ucForceUpdate, void *pParam)
{
	double *d = (double*)pParam;

	if((LastParams.dLevel!=*d) || ucForceUpdate )
	{
		// output parameter to LCD
		if(Params.ucVolt_dB)
			PrintLCD(*d * 0.001f, 0, LCD_TYPE_LEVEL_DB);
		else
			PrintLCD(*d * 0.001f, 0, LCD_TYPE_LEVEL_VOLT);
	}
}

//----------------------------------------------------------------------------
// PeakLevelUpdateDisplayFunction
//
// Gets called periodically to update level display
//
// -> pParam: --
// <- --
//----------------------------------------------------------------------------
void PeakLevelUpdateDisplayFunction(uint8_t ucForceUpdate, void *pParam)
{
	double *d = (double*)pParam;

	if((LastParams.dPeakLevel != *d) || ucForceUpdate )
	{
		PrintLCD(*d * 0.001f, 0, LCD_TYPE_LEVEL_VOLT);
	}
}

//----------------------------------------------------------------------------
// SwitchDBVoltsFunction
//
// Gets called when encoder is pushed while level menu, TRMSC RMS, TRMSC peak
// or similar item was active
//
// -> --
// <- --
//----------------------------------------------------------------------------
void SwitchDBVoltsFunction()
{
	// change "dB" or "Volts" setting
	if(Params.ucVolt_dB)
		Params.ucVolt_dB = 0;
	else
		Params.ucVolt_dB = 1;

	// force update of the parameter
	void (*UpdateFunction)(uint8_t ucForceUpdate, void *pParam) =
		(void*)pgm_read_word(&g_currentMenuItem->pUpdateDisplayFunction);

	void (*pParameter) = (void*)pgm_read_word(&g_currentMenuItem->pParam);

	if((null != UpdateFunction) && (null != pParameter))
		UpdateFunction(1, pParameter);
}

//----------------------------------------------------------------------------
// OffsetEncoderFunction
//
// Gets called when encoder is turned while offset menu item was active
//
// -> iEncoderPos = encoder value difference since last call (positive or
//                  negative)
//    pParam: --
// <- --
//----------------------------------------------------------------------------
void OffsetEncoderFunction(int16_t iEncoderPos,
		void *pParam __attribute__((unused)))
{
	// change value
	Params.iOffset += iEncoderPos;
}

//----------------------------------------------------------------------------
// OffsetUpdateDisplayFunction
//
// Gets called periodically to update offset display
//
// -> pParam: --
// <- --
//----------------------------------------------------------------------------
void OffsetUpdateDisplayFunction(uint8_t ucForceUpdate,
		void *pParam __attribute__((unused)))
{
	if((LastParams.iOffset!=Params.iOffset)||ucForceUpdate)
	{
		// output parameter to LCD
		PrintLCD(0, Params.iOffset, LCD_TYPE_OFFSET);
	}
}

//----------------------------------------------------------------------------
// OffsetEnterFunction
//
// Gets called when encoder is pushed while offset menu item was active
//
// -> --
// <- --
//----------------------------------------------------------------------------
void OffsetEnterFunction()
{
	static int16_t s_iOffsetToggleValue = 0.0f;

	// toggle offset and zero (only via encoder on panel)
	if (pParams->iOffset != 0)
	{
		s_iOffsetToggleValue = pParams->iOffset;
		pParams->iOffset = 0;
	}
	else
	{
		pParams->iOffset = s_iOffsetToggleValue;
	}

	// re-display the parameter by calling the encoder function with
	// argument "0"
	void (*EncoderFunction)(int16_t iEncoderValue) =
		(void*)pgm_read_word(&g_currentMenuItem->pEncoderFunction);
	if(null!=EncoderFunction) EncoderFunction(0);
}

//----------------------------------------------------------------------------
// WaveformUpdateDisplayFunction
//
// Gets called periodically to update waveform display
//
// -> pParam: --
// <- --
//----------------------------------------------------------------------------
void WaveformUpdateDisplayFunction(uint8_t ucForceUpdate,
		void *pParam __attribute__((unused)))
{
	if((LastParams.ucWaveForm!=Params.ucWaveForm)||ucForceUpdate)
	{
		// display new setting
		switch(Params.ucWaveForm)
		{
			case WAVE_OFF:
				LCDOverwrite_P(5, DISPLAY_LINE_1, 3, PSTR("Off"));
				break;
			case WAVE_SINE:
				LCDOverwrite_P(4, DISPLAY_LINE_1, 4, PSTR("Sine"));
				break;
			case WAVE_TRIANGLE:
				LCDOverwrite_P(0, DISPLAY_LINE_1, 8, PSTR("Triangle"));
				break;
			case WAVE_SQUARE:
				LCDOverwrite_P(2, DISPLAY_LINE_1, 6, PSTR("Square"));
				break;
			case WAVE_LOGIC:
				LCDOverwrite_P(2, DISPLAY_LINE_1, 5, PSTR("Logic"));
				Panel_PrintEnter();
				break;
#ifdef COMPILE_WITH_PWM
			case WAVE_PWM:
				LCDOverwrite_P(4, DISPLAY_LINE_1, 3, PSTR("PWM"));
				Panel_PrintEnter();
				break;
#else
			case WAVE_EXT_IN:
				LCDOverwrite_P(1, DISPLAY_LINE_1, 7, PSTR("Ext. In"));
				break;
#endif
		}
	}
}

//----------------------------------------------------------------------------
// WaveformEnterFunction
//
// Gets called when encoder is pushed while waveform menu item was active
//
// -> --
// <- --
//----------------------------------------------------------------------------
void WaveformEnterFunction()
{
	// if "logic" is selected, go to logic settings menu item
	if(WAVE_LOGIC==Params.ucWaveForm)
	{
		g_currentMenuItem = (void*)&SubmenuLogicHi;
		g_ucMenuItemChanged = 1;
	}
}

//----------------------------------------------------------------------------
// Burst_OnOffEncoderFunction
//
// Gets called when encoder is turned while burst on/off submenu item
// was active
//
// -> iEncoderPos = encoder value difference since last call (positive or
//                  negative)
//    pParam: --
// <- --
//----------------------------------------------------------------------------
void Burst_OnOffEncoderFunction(int16_t iEncoderPos,
		void *pParam __attribute__((unused)))
{
	// change setting
	// special handling for manual change with encoder (as opposed to values sent via uart):

	// leave pParams->iBurstOnOff > 0 as is, if iEncoderPos positive
	if ((pParams->iBurstOnOff == 0) && (iEncoderPos > 0))
	{
		pParams->iBurstOnOff = 1;
	}

	// change pParams->iBurstOnOff to 0 if iEncoderPos negative
	if ((pParams->iBurstOnOff != 0) && (iEncoderPos < 0))
	{
		pParams->iBurstOnOff = 0;
	}

}

//----------------------------------------------------------------------------
// Burst_OnOffUpdateDisplayFunction
//
// Gets called periodically to update burst on/off display
//
// -> pParam: --
// <- --
//----------------------------------------------------------------------------
void Burst_OnOffUpdateDisplayFunction(uint8_t ucForceUpdate,
		void *pParam __attribute__((unused)))
{
	if((LastParams.iBurstOnOff!=Params.iBurstOnOff)||ucForceUpdate)
	{
		if(Params.iBurstOnOff)
		{
			LCDOverwrite_P(5, DISPLAY_LINE_1, 2, PSTR("On"));
			Panel_PrintEnter();
		}
		else
		{
			LCDOverwrite_P(4, DISPLAY_LINE_1, 3, PSTR("Off"));
			Panel_PrintEnter();
		}
	}
}


//----------------------------------------------------------------------------
// IEncoderFunction
//
// Gets called when encoder is turned while
//
// -> iEncoderPos = encoder value difference since last call (positive or
//                  negative)
//    pParam: pointer to Params.Burst0 or Params.Burst1 or Params.SweepTime
// <- --
//----------------------------------------------------------------------------
void IEncoderFunction(int16_t iEncoderPos, void *pParam)
{
	int16_t *i = (int16_t*)pParam;

	// change setting
	*i += iEncoderPos;
}

//----------------------------------------------------------------------------
// UCEncoderFunction
//
// Gets called when encoder is turned while in Menu
// for non-negative values
//----------------------------------------------------------------------------
void UCEncoderFunction(int16_t iEncoderPos, void *pParam)
{
	uint8_t *uc = (uint8_t*)pParam;

		// don't make the value "negative"
	if ((*uc == 0) && (iEncoderPos < 0))
		return;

		// change setting
	*uc += iEncoderPos;
}

//----------------------------------------------------------------------------
// TimeUpdateDisplayFunction
//
// Gets called periodically to update time display
//
//    pParam: pointer to Params.Burst0, Params.Burst1, Params.SweepTime,
//            Params.uPermanentTRMSC_Delay or Params.uScrollSpeed
// <- --
//----------------------------------------------------------------------------
void TimeUpdateDisplayFunction(uint8_t ucForceUpdate, void *pParam)
{
	int16_t *i = (int16_t*)pParam;

	if((g_currentMenuItem==&SubmenuSweepTime)&&
			((LastParams.iSweepTime!=pParams->iSweepTime) || ucForceUpdate))
	{
		PrintLCD(0, *i, LCD_TYPE_TIME_100MS);
	}
	else if((LastParams.iBurst0!=pParams->iBurst0)||
		(LastParams.iBurst1!=pParams->iBurst1)||
		(LastParams.iScrollSpeed!=pParams->iScrollSpeed)||
		(LastParams.iPermanentTRMSC_Delay!=pParams->iPermanentTRMSC_Delay)||
		ucForceUpdate)
	{
		PrintLCD(0, *i, LCD_TYPE_TIME_MS);
	}
}

//----------------------------------------------------------------------------
// LogicLevelEncoderFunction
//
// Gets called when encoder is turned while "Logic Hi" or "Logic Lo" submenu
// item was active
//
// -> iEncoderPos = encoder value difference since last call (positive or
//                  negative)
//    pParam = pointer to Logic Lo or Logic Hi
// <- --
//----------------------------------------------------------------------------
void LogicLevelEncoderFunction(int16_t iEncoderPos, void *pParam)
{
	int *i = (int*)pParam;;

	// change value
	*i += iEncoderPos;
}

//----------------------------------------------------------------------------
// LogicLevelUpdateDisplayFunction
//
// Gets called periodically to update logic-hi and logic-lo display
//
// -> pParam: pointer to either LogicLo or LogicHi
// <- --
//----------------------------------------------------------------------------
void LogicLevelUpdateDisplayFunction(uint8_t ucForceUpdate, void *pParam)
{
	int16_t *i = pParam;

	if((LastParams.iLogicHi!=Params.iLogicHi)||
		(LastParams.iLogicLo!=Params.iLogicLo)||ucForceUpdate)
	{
		// output parameter to LCD
		PrintLCD(0, (float)*i, LCD_TYPE_OFFSET);
	}
}

//----------------------------------------------------------------------------
// LogicLevelEnterFunction
//
// Gets called when encoder is pushed while logic hi or logic lo menu item
// was active
//
// -> --
// <- --
//----------------------------------------------------------------------------
void LogicLevelEnterFunction()
{
	Params.iLogicHi = 5000;
	Params.iLogicLo = 0;
}

//----------------------------------------------------------------------------
// TRMSC_RangeUpdateDisplayFunction
//
// Gets called periodically to update TRMSC range display
//
// -> pParam: --
// <- --
//----------------------------------------------------------------------------
void TRMSC_RangeUpdateDisplayFunction(uint8_t ucForceUpdate,
		void *pParam __attribute__((unused)))
{
	if((LastParams.ucRange!=Params.ucRange)||ucForceUpdate)
	{
		// display new setting
		switch(Params.ucRange)
		{
			case RANGE_100mV:
				LCDOverwrite_P(2, DISPLAY_LINE_1, 6, PSTR("100 mV"));
				break;
			case RANGE_1V:
				LCDOverwrite_P(5, DISPLAY_LINE_1, 3, PSTR("1 V"));
				break;
			case RANGE_10V:
				LCDOverwrite_P(4, DISPLAY_LINE_1, 4, PSTR("10 V"));
				break;
			case RANGE_100V:
				LCDOverwrite_P(3, DISPLAY_LINE_1, 5, PSTR("100 V"));
				break;
			case RANGE_AUTO:
				LCDOverwrite_P(4, DISPLAY_LINE_1, 4, PSTR("auto"));
				break;
		}
	}
}

//----------------------------------------------------------------------------
// TRMSC_PermanentUpdateDisplayFunction
//
// Gets called periodically to update TRMSC permanent display
//
// -> pParam: --
// <- --
//----------------------------------------------------------------------------
void TRMSC_PermanentUpdateDisplayFunction(uint8_t ucForceUpdate,
		void *pParam __attribute__((unused)))
{
	if((LastParams.ucPermanentTRMSC!=Params.ucPermanentTRMSC)||ucForceUpdate)
	{
		// display new setting
		switch(Params.ucPermanentTRMSC)
		{
		case PERMANENT_TRMSC_OFF:
			LCDOverwrite_P(5, DISPLAY_LINE_1, 3, PSTR("Off"));
			break;
		case PERMANENT_TRMSC_RMS:
			LCDOverwrite_P(5, DISPLAY_LINE_1, 3, PSTR("RMS"));
			break;
		case PERMANENT_TRMSC_PEAK:
			LCDOverwrite_P(4, DISPLAY_LINE_1, 4, PSTR("Peak"));
			break;
		}
	}
}

//----------------------------------------------------------------------------
// TRMSC_LevelUpdateDisplayFunction
//
// Gets called periodically to update TRMSC level display
//
// -> pParam: pointer to double RMS or peak
// <- --
//----------------------------------------------------------------------------
void TRMSC_LevelUpdateDisplayFunction(uint8_t ucForceUpdate, void *pParam)
{
	char s[16];
	double *d = pParam;
	double dB;

	if((g_dLastTRMSC_RMS!=g_dTRMSC_RMS)||
		(g_dLastTRMSC_Peak!=g_dTRMSC_Peak)||
		ucForceUpdate)
	{
		if(Status.OverVolt||(*d<0.0f))
		{
			LCDOverwrite_P(0, DISPLAY_LINE_1, 8, PSTR("OVERLOAD"));
		}
		else
		{
			// output parameter to LCD
			if(	Params.ucVolt_dB  &&
				( (g_currentMenuItem == (void*)&SubmenuTRMSC_RMS) ||
				((Params.ucPermanentTRMSC == PERMANENT_TRMSC_RMS) && (g_currentMenuItem != (void*)&SubmenuTRMSC_Peak)) ))
			{
				//   9.99: | +9.99dB|
				//  -9.99: | -9.99dB|
				//  22.22: | +22.2dB|
				// -22.22: | -22.2dB|
				dB = (20 * log10( ( (*d < 0.000025f) ? 0.000025f : *d) / 774.597));
				if ((dB >= 10.0f) || (dB <= -10.0f))
				{
					sprintf_P(s, PSTR("\x04%+2.1fdB"), dB);
				}
				else
				{
					sprintf_P(s, PSTR("\x04%+1.2fdB"), dB);
				}
			}
			else
			{
				if(*d<1000.0f)
			    	sprintf_P(s, PSTR("\x04%5.1fmV"), *d);
			    else
			       	sprintf_P(s, PSTR("\x04%6.3fV"), *d/1000.0f);
			}
			LCDWrite(0, DISPLAY_LINE_1, 8, s);
		}
		g_dLastTRMSC_RMS = g_dTRMSC_RMS;
		g_dLastTRMSC_Peak = g_dTRMSC_Peak;
	}
}

//----------------------------------------------------------------------------
// SweepModeUpdateDisplayFunction
//
// Gets called periodically to update sweep mode display
//
// -> pParam: --
// <- --
//----------------------------------------------------------------------------
void SweepMenuUpdateDisplayFunction(uint8_t ucForceUpdate,
		void *pParam __attribute__((unused)))
{
	if((LastParams.ucSweepMenu!=Params.ucSweepMenu)||ucForceUpdate)
	{
		// display new setting
		switch(Params.ucSweepMenu)
		{
			case SWEEP_OFF:
				LCDOverwrite_P(4, DISPLAY_LINE_1, 3, PSTR("Off"));
				Panel_PrintEnter();
				break;
			case SWEEP_ON:
				LCDOverwrite_P(5, DISPLAY_LINE_1, 2, PSTR("On"));
				Panel_PrintEnter();
				break;
		}
	}
}

//----------------------------------------------------------------------------
// SweepMarkerModeUpdateDisplayFunction
//
// Gets called periodically to update sweep mode display
//
// -> pParam: --
// <- --
//----------------------------------------------------------------------------
void SweepMarkerModeUpdateDisplayFunction(uint8_t ucForceUpdate,
		void *pParam __attribute__((unused)))
{
	if((LastParams.ucSweepMarkerMode!=Params.ucSweepMarkerMode)||ucForceUpdate)
	{
		// display new setting
		switch(Params.ucSweepMarkerMode)
		{
			case MARKER_ABSOLUTE:
				LCDOverwrite_P(0, DISPLAY_LINE_1, 7, PSTR("absolute"));
				Panel_PrintEnter();
				break;
			case MARKER_RELATIVE:
				LCDOverwrite_P(0, DISPLAY_LINE_1, 7, PSTR("relative"));
				Panel_PrintEnter();
				break;
		}
	}
}

//----------------------------------------------------------------------------
// SweepSlopeUpdateDisplayFunction
//
// Gets called periodically to update sweep mode display
//
// -> pParam: --
// <- --
//----------------------------------------------------------------------------
void SweepSlopeUpdateDisplayFunction(uint8_t ucForceUpdate,
		void *pParam __attribute__((unused)))
{
	if((LastParams.ucSweepSlope!=Params.ucSweepSlope)||ucForceUpdate)
	{
		// display new setting
		switch(Params.ucSweepSlope)
		{
			case SWEEP_UP:
				LCDOverwrite_P(2, DISPLAY_LINE_1, 5, PSTR("Up   "));
				Panel_PrintEnter();
				break;
			case SWEEP_DOWN:
				LCDOverwrite_P(2, DISPLAY_LINE_1, 5, PSTR("Down "));
				Panel_PrintEnter();
				break;
			case SWEEP_UPDOWN:
				LCDOverwrite_P(0, DISPLAY_LINE_1, 7, PSTR("Up+Down"));
				Panel_PrintEnter();
				break;
		}
	}
}

//----------------------------------------------------------------------------
// SweepModeUpdateDisplayFunction
//
// Gets called periodically to update sweep formula display
//
// -> pParam: --
// <- --
//----------------------------------------------------------------------------
void SweepModeUpdateDisplayFunction(uint8_t ucForceUpdate,
		void *pParam __attribute__((unused)))
{
	if((LastParams.ucSweepMode!=Params.ucSweepMode)||ucForceUpdate)
	{
		// display new setting
		switch(Params.ucSweepMode)
		{
			case SWEEP_LINEAR:
				LCDOverwrite_P(2, DISPLAY_LINE_1, 6, PSTR("linear"));
				break;
			case SWEEP_LOG:
				LCDOverwrite_P(2, DISPLAY_LINE_1, 3, PSTR("log"));
				break;
		}
	}
}
//----------------------------------------------------------------------------
// SweepMarkerUpdateDisplayFunction
//
// Gets called periodically to update sweep formula display
//
// -> pParam: --
// <- --
//----------------------------------------------------------------------------
void SweepMarkerUpdateDisplayFunction(uint8_t ucForceUpdate,
		void *pParam __attribute__((unused)))
{
	if((LastParams.ucSweepMarker!=Params.ucSweepMarker)||ucForceUpdate)
	{
		// display new setting
		switch(Params.ucSweepMarker)
		{
			case MARKER_OFF:
				LCDOverwrite_P(2, DISPLAY_LINE_1, 3, PSTR("Off"));
				break;
			case MARKER_DECADE:
				LCDOverwrite_P(2, DISPLAY_LINE_1, 6, PSTR("decade"));
				break;
			case MARKER_OCTAVE:
				LCDOverwrite_P(2, DISPLAY_LINE_1, 6, PSTR("octave"));
				break;
/*			case MARKER_ONEPONE:
				LCDOverwrite_P(2, DISPLAY_LINE_1, 4, PSTR("*1.1"));
				break;
*/		}
	}
}

//----------------------------------------------------------------------------
// SettingsSaveUpdateDisplayFunction
//
// Gets called periodically to update TRMSC range display
//
// -> pParam: --
// <- --
//----------------------------------------------------------------------------
void SettingsSaveUpdateDisplayFunction(uint8_t ucForceUpdate,
		void *pParam __attribute__((unused)))
{
	if(ucForceUpdate||(Params.ucAutosave!=LastParams.ucAutosave))
	{
		// display new setting
		switch(Params.ucAutosave)
		{
			case 0:
				LCDOverwrite_P(4, DISPLAY_LINE_1, 3, PSTR("now"));
				Panel_PrintEnter();
				break;
			case 1:
			default:
				LCDOverwrite_P(0, DISPLAY_LINE_1, 8, PSTR("auto 30s"));
				break;
		}
	}
}

//----------------------------------------------------------------------------
// SettingsSaveEnterFunction
//
// Gets called when encoder is pushed while waveform menu item was active
//
// -> --
// <- --
//----------------------------------------------------------------------------
void SettingsSaveEnterFunction(void)
{
	if(0==Params.ucAutosave)
	{
		LCDOverwrite_P(1, DISPLAY_LINE_0, 6, PSTR("Saving"));
		LCDClearLine(1);
		wait_ms(1000);
		eeprom_write_block(&Params, EEPROM_PARAMS, sizeof(Params));
		LCDOverwrite_P(3, DISPLAY_LINE_0, 2, PSTR("OK"));
		g_ucMenuItemChanged = 1;
		wait_ms(3000);
		g_currentMenuItem = (void*)&MenuSettings;
	}
}



//----------------------------------------------------------------------------
// PrintLCD
//
// Output various values to the LCD.
//
// -> f = offset voltage
//    uType = LCD_TYPE_OFFSET: print offset voltage in mV, V
//          = LCD_TYPE_LEVEL_VOLT: print level voltage in mV, V
//          = LCD_TYPE_LEVEL_DB: print level voltage in dB
//          = LCD_TYPE_FREQUENCY: output frequency in Hz
//          = LCD_TYPE_TIME_MS: output time in milliseconds
// <- --
//----------------------------------------------------------------------------
void PrintLCD(double f, int16_t i, uint8_t uType)
{
	char s[10];
	double dB;

	// TODO: f entfernen, wenn PrintLCD auf Integer umgestellt ist

	// isolate fractional part of i without sign
	int16_t j = ((i<0)?-i:i) % 1000;

    // 2nd line
    switch(uType)
	{
	case LCD_TYPE_OFFSET:
//		if(f<-10.0f) f = -10.0f;
//		if(f>10.0f) f = 10.0f;
		//   0.000: |    0 mV|
		//   0.999: | +999 mV|
		//  -0.999: | -999 mV|
		//   0.099: |  +99 mV|
		//  -0.099: |  -99 mV|
		//   0.009: |   +9 mV|
		//  -0.009: |   -9 mV|
		//   9.999: | +9.999V|
		//  -9.999: | -9.999V|
		//  10.230: |+10.230V|
		// -10.230: |-10.230V|
		if(0==i)
	    	sprintf_P(s, PSTR("    0 mV"));
	    else if((i<1000)&&(i>-1000))
	    	sprintf_P(s, PSTR("%+5d mV"), i);
	    else
	       	sprintf_P(s, PSTR("%+3d.%03dV"), i/1000, j);
//			sprintf_P(s, PSTR("%+7.3fV"), f/1000.0f);
	    break;

	case LCD_TYPE_LEVEL_VOLT:
//		if(f>10.0f) f = 10.0f;
//		if(f<0.000025f) f = 0.000025f;
		//  0.009125: | 9.125mV|
		//  0.099125: |99.125mV|
		//  0.199125: |199.13mV|
		//  0.200000: |   200mV|
		//  0.999000: |   999mV|
		//  9.999000: | 9.999 V|
		// 10.000000: |10.000 V|
		if(f<0.1f)
	    	sprintf_P(s, PSTR("%6.3fmV"), f*1000.0f);
		else if(f<=0.2f)
	    	sprintf_P(s, PSTR("%6.2fmV"), f*1000.0f);
		else if(f<1.0f)
	    	sprintf_P(s, PSTR("   %dmV"), (int)(f*1000.0f));
	    else
	       	sprintf_P(s, PSTR("%7.3fV"), f);
	    break;

	case LCD_TYPE_LEVEL_DB:
		if(f<0.000025f) f = 0.000025f;
//		if(f>10.0f) f = 10.0f;
		//   9.99: | +9.99dB|
		//  -9.99: | -9.99dB|
		//  22.22: |+22.22dB|
		// -22.22: |-22.22dB|
		dB = (20 * log10(f/0.774597));
		sprintf_P(s, PSTR("%+6.2fdB"), dB);
		break;

	case LCD_TYPE_FREQUENCY:
		if(f<0.015625f) f = 0.015625f;
		if(f>MAX_FREQUENCY) f = MAX_FREQUENCY;

		if(f<10.0f)
			sprintf_P(s, PSTR("%6.3fHz"), f);
		else if(f<100.0f)
			sprintf_P(s, PSTR("%6.2fHz"), f);
		else if(f<1000.0f)
			sprintf_P(s, PSTR("%6.1fHz"), f);
		else if(f<10000.0f)
			sprintf_P(s, PSTR("%5.3fkHz"), f/1000.0f);
		else if(f<100000.0f)
			sprintf_P(s, PSTR("%5.2fkHz"), f/1000.0f);
		else
			sprintf_P(s, PSTR("%5.1fkHz"), f/1000.0f);
		break;

	case LCD_TYPE_TIME_MS:
		if(i<0) i = 0;
		sprintf_P(s, PSTR("%5d ms"), i);
		break;

	case LCD_TYPE_TIME_100MS:
		if(i<0) i = 0;
       	sprintf_P(s, PSTR("%4d.%1d s"), i/1000, j/100);
//		sprintf_P(s, PSTR("%6.1f s"), i/1000.0f, j);
       	break;

	case LCD_TYPE_FLOAT:
			sprintf_P(s, PSTR("%3.2f    "), f);
		break;
	}

	LCDWrite(0, DISPLAY_LINE_1, 8, s);
	
	
}


//----------------------------------------------------------------------------
// findMenuEntry
//
// -> ucIdentifier
//
// <- g_currentMenuItem
//----------------------------------------------------------------------------


menu_entry_t *findMenuEntry(uint8_t ucIdentifier)
{
	menu_entry_t *foundEntry;
	int iSafetyCntr = 0; // avoid endless loops if lists are not properly maintained or I made an error iterating through it ;-)

	if (pgm_read_byte(&g_currentMenuItem->ucIdentifier) == ucIdentifier)
	{
		return g_currentMenuItem;
	} else {
	    foundEntry = (menu_entry_t *) pgm_read_word(&g_currentMenuItem->pNext);
	    do
		{
			if (pgm_read_byte(&foundEntry->ucIdentifier) == ucIdentifier)
			{
				return foundEntry;
			} else {
			    foundEntry = (menu_entry_t *)pgm_read_word(&foundEntry->pNext);
			}
			if (iSafetyCntr ++ > 100) break;
		} while (((menu_entry_t *)(pgm_read_word(&foundEntry->pNext)) != g_currentMenuItem) && ((menu_entry_t *)(pgm_read_word(&foundEntry->pNext)) != null));
	}
	return g_currentMenuItem;
}

// TODO habe es kurzfristig in panel.c gepackt, muss zurck 
const prog_char VersStrShort[] = "DDS C1.0"; // 8chars max

void Panel_SplashScreen(void)
{
	char s[9];
	
    
#ifdef COMPILE_WITH_DISPLAY204
	// DDS in big
	LCDWrite_P(0, 0, 12, PSTR("\x1f\x1f\x02\x20\x1f\x1f\x02\x20\x04\x1f\x1f\x20"));
	LCDWrite_P(0, 1, 12, PSTR("\x1f\x20\x1f\x20\x1f\x20\x1f\x20\x05\x07\x20\x20"));
	LCDWrite_P(0, 2, 12, PSTR("\x1f\x20\x1f\x20\x1f\x20\x1f\x20\x20\x06\x02\x20"));
	LCDWrite_P(0, 3, 12, PSTR("\x1f\x1f\x03\x20\x1f\x1f\x03\x20\x1f\x1f\x03\x20"));

	// Infos in small
    sprintf_P(s, VersStrShort);
    LCDWrite(12, 1, 8, s);
    sprintf_P(s, PSTR("Adr  %-3d"), g_ucSlaveCh);
    LCDWrite(12, 2, 8, s);
    	
	LCDWrite_P(12, 3, 3, PSTR("RC0"));
#else
	sprintf_P(s, VersStrShort);
    LCDWrite(0, 0, 8, s);
    sprintf_P(s, PSTR("Adr  %-3d"), g_ucSlaveCh);
    LCDWrite(0, 1, 8, s);
#endif
}

void Panel_PrintEnter(void)
{
#ifdef COMPILE_WITH_DISPLAY204
	LCDWrite_P(7, 2, 1, PSTR("\x10"));
#else
	LCDWrite_P(7, 1, 1, PSTR("\x05"));
#endif

}


// works properly only if jobPanel runs before jobExecute
void Panel_UpdateStatusLine(void)
{
#ifdef COMPILE_WITH_DISPLAY204
    if (LastParams.iBurstOnOff != Params.iBurstOnOff)
    {
        if (Params.iBurstOnOff == 0)
            LCDWrite_P(15, 0, 1, PSTR("\x20"));
        else
            LCDWrite_P(15, 0, 1, PSTR("B"));

    }

    if (LastParams.ucSweepMenu != Params.ucSweepMenu)
    {
        if (Params.ucSweepMenu == SWEEP_OFF)
            LCDWrite_P(17, 0, 1, PSTR("\x20"));
        else
            LCDWrite_P(17, 0, 1, PSTR("S"));
    }

    if (LastParams.ucWaveForm != Params.ucWaveForm)

    {
        switch (Params.ucWaveForm)
        {
            // Darstellung entweder Asci oder schicke Zeichen aus ea dip 204
            case WAVE_OFF:
                LCDWrite_P(19, 0, 1, PSTR("O"));
                break;
            case WAVE_SINE:
                LCDWrite_P(19, 0, 1, PSTR("\xce"));
                break;
            case WAVE_TRIANGLE:
                LCDWrite_P(19, 0, 1, PSTR("\xc1"));
                break;
            case WAVE_SQUARE:
                LCDWrite_P(19, 0, 1, PSTR("\xc2"));
                break;
            case WAVE_LOGIC:
                LCDWrite_P(19, 0, 1, PSTR("L"));
                break;

#ifdef COMPILE_WITH_PWM
                case WAVE_PWM:
                LCDWrite_P(19,0,1,PSTR("P"));
                break;
#else
            case WAVE_EXT_IN:
                LCDWrite_P(19, 0, 1, PSTR("E"));
                break;
#endif

            default:
                LCDWrite_P(19, 0, 1, PSTR("F"));
        }
    }
#else
#endif
}

